{******************************************************************************}
{  History Component                                                           }
{  Copyright (C) 2001-2002 Dan Strandberg                                      }
{------------------------------------------------------------------------------}
{                                                                              }
{  Version     Author      Date        Description                             }
{  1.0.0.0      Dan        25/08/01    Initial version                         }
{  1.1.0.0      Dan        24/01/02    Changed various stuff                   }
{  1.1.1.0      Dan        07/03/02    Added "IfExistsPlaceFirst"              }
{  1.1.2.0      Dan        17/07/02    IfExistsPlaceFirst defaults to true     }
{                                                                              }
{******************************************************************************}
unit History;

interface

uses
  Windows, Messages, SysUtils, Classes, IniFiles;

type
  THistory = class(TComponent)
  private
    FMaxItems: Integer;
    FItems   : TstringList;
    FPlaceFirst: Boolean;
    FOnChange: TNotifyEvent;
    { Private declarations }
  protected
    function GetItem(Index: integer): string;
    function GetItemCount: Integer;
    procedure SetMaxItems(Value: integer);
    procedure DoChange; virtual;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddItem(Item: string);
    procedure Clear;
    procedure DeleteItem(Index: Integer); overload;
    procedure DeteteItem(Item: string); overload;
    procedure LoadFromIni(IniFile: string; Section: string = 'history'); overload;
    procedure LoadFromIni(IniFile: TIniFile; Section: string = 'history'); overload;
    procedure SaveToIni(IniFile: string; Section: string = 'history'); overload;
    procedure SaveToIni(IniFile: TIniFile; Section: string = 'history'); overload;
    property Count: integer read GetItemCount;
    property Items[Index: Integer]: string read GetItem;
    { Public declarations }
  published
    property MaxItems: integer read FMaxItems write SetMaxItems;
    property IfExistsPlaceFirst: Boolean read FPlaceFirst write FPlaceFirst;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    { Published declarations }
  end;

procedure Register;

implementation

constructor THistory.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMaxItems := 5;
  IfExistsPlaceFirst := True;
  FItems := TstringList.Create;
end;

destructor THistory.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure THistory.SetMaxItems(Value: integer);
begin
  if Value <> FMaxItems then
  begin
    if Value < FItems.Count then
    begin
      while FItems.Count > FMaxItems do
        FItems.Delete( FItems.Count -1 );
      DoChange;
    end;
    FMaxItems := Value;
  end;
end;

procedure THistory.AddItem(Item: string);
begin
  if Item <> '' then
  begin
    if FItems.IndexOf(Item) > -1 then
    begin
      if FPlaceFirst then // Item is already on the list, move to the top
        FItems.Move(FItems.IndexOf(Item), 0);
    end else
    begin
      // Add item.
      FItems.Insert(0, Item);
      while FItems.Count > FMaxItems do
        FItems.Delete(FItems.Count -1);
    end;
    DoChange;
  end;
end;

procedure THistory.Clear;
begin
  FItems.Clear;
  DoChange;
end;

procedure THistory.DeleteItem(Index: Integer);
begin
  if (Index < FItems.Count) and (Index >= 0) then
  begin
    FItems.Delete(Index);
    DoChange;
  end;
end;

procedure THistory.DeteteItem(Item: string);
begin
  DeleteItem(FItems.IndexOf(Item));
end;

function THistory.GetItem(Index: integer): string;
begin
  Result := '';
  if (Index < FItems.Count) and (Index >= 0) then
    Result := FItems.strings[Index]
  else
    raise Exception.CreateFmt('Index out of bounds (%d)', [Index]);
end;

function THistory.GetItemCount: Integer;
begin
  Result := FItems.Count;
end;

procedure THistory.DoChange;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure THistory.LoadFromIni(IniFile: string; Section: string);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(IniFile);
  try
    LoadFromIni(Ini, Section);
  finally
    FreeAndNil(Ini);
  end;
end;

procedure THistory.LoadFromIni(IniFile: TIniFile; Section: string);
var
  i:integer;
  s:string;
begin
  if not Assigned(IniFile) then
    raise Exception.Create('IniFile not assigned');

  if Section = '' then
    raise Exception.Create('Section is not set!');

  try
    try
      FItems.Clear;
      for i := 0 to FMaxItems-1 do
      begin
        s := IniFile.Readstring(Section, IntToStr(i), '');
        if s <> '' then
          FItems.Add(s);
      end;
    except
      raise
    end;
  finally
    DoChange;
  end;
end;

procedure THistory.SaveToIni(IniFile: string; Section: string);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(IniFile);
  try
    SaveToIni(Ini, Section);
  finally
    FreeAndNil(Ini);
  end;
end;

procedure THistory.SaveToIni(IniFile: TIniFile; Section: string);
var
  i:integer;
begin
  if not Assigned(IniFile) then
    raise Exception.Create('IniFile not assigned');

  if Section = '' then
    raise Exception.Create('Section is not set!');

  try
    IniFile.EraseSection(Section);
    for i := 0 to FItems.Count-1 do
      IniFile.Writestring(Section, IntToStr(i), FItems.strings[i]);
  except
    raise
  end;
end;

procedure Register;
begin
  RegisterComponents('Game-Editing.net', [THistory]);
end;

end.
